USTechFirms = {
    'head': ['symbol', 'companyName', 'industry', 'ceo', 'sector', 'price', 'mktCap'],
    'rows': [
        ['AAPL', 'Apple Inc.', 'Computer Hardware', 'Timothy D. Cook', 'Technology', 288.08, '1260488230000'],
        ['MSFT', 'Microsoft Corporation', 'Application Software', 'Satya Nadella', 'Technology', 168.07,
         '1223232060000'],
        ['FB', 'Facebook Inc.', 'Online Media', 'Mark Zuckerberg', 'Technology', 196.77, '560883040000'],
        ['NVDA', 'NVIDIA Corporation', 'Semiconductors', 'Jen-Hsun Huang', 'Technology', 262.05, '160374587000'],
        ['AMZN', 'Amazon.com Inc.', 'Online Media', 'Jeffrey P. Bezos', 'Technology', 1972.74, '982049690000'],
        ['GOOG', 'Alphabet Inc.', 'Online Media', 'Larry Page', 'Technology', 1388.45, '953677710000'],
        ['AMD', 'Advanced Micro Devices Inc.', 'Semiconductors', 'Lisa T. Su', 'Technology', 47.57, '55640728000'],
        ['QCOM', 'QUALCOMM Incorporated', 'Semiconductors', 'Steven M. Mollenkopf', 'Technology', 80.22, '88466620000'],
        ['CRM', 'Salesforce.com Inc', 'Application Software', 'Marc Benioff', 'Technology', 181.27, '160786498000'],
        ['INTC', 'Intel Corporation', 'Semiconductors', 'Brian M. Krzanich', 'Technology', 59.73, '245220917000'],
        ['TSM', 'Taiwan Semiconductor Manufacturing Company Ltd.', 'Semiconductors', 'Mark Liu / C.C Wei', 'Technology',
         53.86, '271774351000']
    ],
    'title': ''
}



def fibonacci_generator():
    prev, curr = 0, 1
    while True:
        yield curr
        prev, curr = curr, prev + curr


def fibonacci(n):
    # vec = []
    gen = fibonacci_generator()
    # while (n := n - 1) >= 0:
    #     vec.append(next(gen))
    # return vec
    return [next(gen) for _ in range(n)]



actresses = [
    'Kathy',
    'Anjelica',
    'Julia',
    'Meryl',
    'Joanne',
    'Jodie',
    'Geena',
    'Laura',
    'Bette',
    'Susan',
    'Emma',
    'Catherine',
    'Mary',
    'Michelle',
    'Susan',
    'Holly',
    'Angela',
    'Stockard',
    'Emma',
    'Debra',
    'Jessica',
    'Jodie',
    'Miranda',
    'Winona',
    'Susan',
    'Susan',
    'Elisabeth',
    'Sharon',
    'Meryl',
    'Emma',
    'Frances',
    'Brenda',
    'Diane',
    'Kristin',
    'Emily',
    'Helen',
    'Helena',
    'Julie',
    'Judi',
    'Kate',
    'Gwyneth',
    'Cate',
    'Fernanda',
    'Meryl',
    'Emily',
    'Hilary',
    'Annette',
    'Janet',
    'Julianne',
    'Meryl',
    'Julia',
    'Joan',
    'Juliette',
    'Ellen',
    'Laura',
    'Halle',
    'Judi',
    'Nicole',
    'Sissy',
    'Renée',
    'Nicole',
    'Salma',
    'Diane',
    'Julianne',
    'Renée',
    'Charlize',
    'Keisha',
    'Diane',
    'Samantha',
    'Naomi',
    'Hilary',
    'Annette',
    'Catalina',
    'Imelda',
    'Kate',
    'Reese',
    'Judi',
    'Felicity',
    'Keira',
    'Charlize',
    'Helen',
    'Penélope',
    'Judi',
    'Meryl',
    'Kate',
    'Marion',
    'Cate',
    'Julie',
    'Laura',
    'Ellen',
    'Kate',
    'Anne',
    'Angelina',
    'Melissa',
    'Meryl',
    'Sandra',
    'Helen',
    'Carey',
    'Gabourey',
    'Meryl',
]



TopBoxOffice = {
    'head': ['name', 'boxoffice', 'budget', 'year', 'director', 'producer', 'writer', 'based_on'],
    'rows': [
        ['Avatar', '2,788', 244, '2009', 'James Cameron', ['James Cameron', 'Jon Landau'], 'James Cameron', ''],
        ['Titanic', '2,187', 200, '1997', 'James Cameron', ['James Cameron', 'Jon Landau'], 'James Cameron', ''],
        ['Star Wars: The Force Awakens', '2,068', 306, '2015', 'J. J. Abrams',
         ['Kathleen Kennedy', 'J. J. Abrams', 'Bryan Burk'], ['Lawrence Kasdan', 'J. J. Abrams', 'Michael Arndt'],
         'George Lucas'],
        ['Avengers: Infinity War', '2,048', 400, '2018', ['Anthony Russo', 'Joe Russo'], 'Kevin Feige',
         ['Christopher Markus', 'Stephen McFeely'], ['Stan Lee', 'Jack Kirby']],
        ['Jurassic World', '1,672', 150, '2015', 'Colin Trevorrow', ['Frank Marshall', 'Patrick Crowley'],
         ['Rick Jaffa', 'Amanda Silver', 'Derek Connolly', 'Colin Trevorrow'], 'Michael Crichton'],
        ['The Avengers', '1,519', 220, '2012', 'Joss Whedon', 'Kevin Feige', ['Joss Whedon', 'Zak Penn'],
         ['Stan Lee', 'Jack Kirby']],
        ['Furious 7', '1,516', 190, '2015', 'James Wan', ['Neal H. Moritz', 'Vin Diesel', 'Michael Fottrell'],
         'Chris Morgan', 'Gary Scott Thompson'],
        ['Avengers: Age of Ultron', '1,405', 495, '2015', 'Joss Whedon', 'Kevin Feige', 'Joss Whedon',
         ['Stan Lee', 'Jack Kirby']],
        ['Black Panther', '1,347', 210, '2018', 'Ryan Coogler', 'Kevin Feige', ['Ryan Coogler', 'Joe Robert Cole'],
         ['Stan Lee', 'Jack Kirby']],
        ['Harry Potter & Deathly Hallows II', '1,342', 250, '2011', 'David Yates',
         ['David Heyman', 'David Barron', 'J. K. Rowling'], 'Steve Kloves', 'J. K. Rowling'],
        ['Star Wars: The Last Jedi', '1,333', 317, '2017', 'Rian Johnson', ['Kathleen Kennedy', 'Ram Bergman'],
         'Rian Johnson', 'George Lucas'],
        ['Jurassic World: Fallen Kingdom', '1,309', 187, '2018', 'J. A. Bayona',
         ['Frank Marshall', 'Patrick Crowley', 'Belén Atienza'], ['Derek Connolly', 'Colin Trevorrow'],
         'Michael Crichton'],
        ['Frozen', '1,290', 150, '2013', ['Chris Buck', 'Jennifer Lee'], 'Peter Del Vecho',
         ['Chris Buck', 'Jennifer Lee', 'Shane Morris'], ''],
        ['Beauty and the Beast', '1,263', 255, '2017', 'Bill Condon', ['David Hoberman', 'Todd Lieberman'],
         ['Stephen Chbosky', 'Evan Spiliotopoulos'], 'Disney\'s'],
        ['Incredibles 2', '1,242', 200, '2018', 'Brad Bird', ['John Walker', 'Nicole Paradis Grindle'], 'Brad Bird',
         ''],
        ['The Fate of the Furious', '1,239', 250, '2017', 'F. Gary Gray',
         ['Neal H. Moritz', 'Vin Diesel', 'Michael Fottrell', 'Chris Morgan'], 'Chris Morgan', 'Gary Scott Thompson'],
        ['Iron Man 3', '1,215', 200, '2013', 'Shane Black', 'Kevin Feige', ['Drew Pearce', 'Shane Black'],
         ['Stan Lee', 'Don Heck', 'Larry Lieber', 'Jack Kirby']],
        ['Minions', '1,159', 74, '2015', ['Pierre Coffin', 'Kyle Balda'], ['Chris Meledandri', 'Janet Healy'],
         'Brian Lynch', 'Sergio Pablos'],
        ['Captain America: Civil War', '1,153', 250, '2016', ['Anthony Russo', 'Joe Russo'], 'Kevin Feige',
         ['Christopher Markus', 'Stephen McFeely'], ['Joe Simon', 'Jack Kirby']],
        ['Aquaman', '1,139', 200, '2018', 'James Wan', ['Peter Safran', 'Rob Cowan'],
         ['David Leslie Johnson-McGoldrick', 'Will Beall', 'Geoff Johns', 'James Wan'], 'Mort Weisinger']]
}



from .absolute_mirror import absolute_mirror
from .compound_interest import compound_interest
from .fibonacci import fibonacci
from .hansen_samuelson import hansen_samuelson
from .leonardo import leonardo
from .primes import primes



def is_prime(n):
    if n < 2: return False
    if not n % 2: return n == 2
    if not n % 3: return n == 3
    d = 5
    while d * d <= n:
        if not n % d: return False
        d += 2
        if not n % d: return False
        d += 4
    return True


def primes(size):
    vec, num = [], 1
    while True:
        if is_prime(num := num + 1):
            vec.append(num)
            if len(vec) >= size: break
    return vec



# copyright: hbr.org
# link: https://hbr.org/visual-library/2018/03/how-marketing-moved-from-the-mass-market-era-to-the-era-of-relevance

# head = ['Era','Decade','Technology enabler','Perf indicator','Market approach','Management focus']
MarketingMovement = [
    ['Mass-market', 1960, 'Mass production', 'Volume', 'Mass appeal', 'Product & scale'],
    ['Segment', 1980, 'Market research', 'Purchase funnel', 'Segmentation', 'Channel & scale'],
    ['Customer', 1990, 'Enterprise IT', 'Customer lifetime value', 'Proposition innovation', 'Channel & relationship'],
    ['Loyalty', 2010, 'Advanced CRM', 'Customer retention', 'Tailored incentives', 'Experience & relationship'],
    ['Relevance', 2020, 'Digitization of everything', 'Customer attraction', 'Personalization',
     'Experience & personality'],
]



car_plants = {
    'SantAgata': 'Lamborghini',
    'Angelholm': 'Koenigsegg',
    'Molsheim': 'Bugatti',
    'Gaydon': 'Aston Martin',
    'Maranello': 'Ferrari',
    'Stuttgart': 'Porsche',
    'Modena': 'Pagani',
    'Neckarsulm': 'Audi',
    'Crewe': 'Bentley',
    'Woking': 'McLaren',
    'Hethel': 'Lotus',
}



military_robots = {
    'MechanicalHound': 'Fahrenheit 451',
    'AMEE': 'Red Planet',
    'SIMON': 'Lara Croft Tomb Raider',
    'ED209': 'RoboCop',
    'BattleRobotT1': 'Terminator 3 Rise of the Machines',
    'Johnny5': 'Short Circuit',
    'MARK': 'Hardware',
    'EDI': 'Stealth',
    'VTOLCraftT1': 'Terminator 3 Rise of the Machines',
    'CyberdyneT800': 'Terminator series',
    'EosB1': 'Star Wars Ep. I/II/III',
    'EosB2': 'Star Wars Ep. II/III',
    'HolowanMagnaGuards': 'Star Wars Ep. III',
    'Decepticons': 'Transformers',
    'HectorModel': 'Saturn 3',
    'STAR': 'The Black Hole',
    'CylonCenturion': 'Battlestar Galactica',
    'Protectron': 'Fallout',
    'SentryBot': 'Fallout',
    'LibertyPrime': 'Fallout',
    'CyberdyneT1000': 'Terminator 2 Judgment Day',
    'Terminatrix': 'Terminator 3 Rise of the Machines',
    'Synth': 'Fallout',
    'Sentinels': 'The Matrix series',
    'RobotB9': 'Lost in Space',
    'Droideka': 'Star Wars Ep. I,II,III',
    'R2D2': 'Star Wars series',
    'VINCENT': 'The Black Hole',
    'BOB': 'The Black Hole',
    'Maximilian': 'The Black Hole',
    'MisterGusty': 'Fallout series',
    'SuperSentinels': 'Halo 1/2/3',
    'Screamers': 'Screamers',
    'APU': 'The Matrix Revolutions',
    'IronManSuit': 'Iron Man',
    'AMP': 'Avatar',
    'MANTIS': 'M.A.N.T.I.S.',
    'PoweredInfantryArmorT51': 'Fallout series',
}



film_actresses = {
    'Misery': 'Kathy Bates',
    'The Grifters': 'Anjelica Huston',
    'Pretty Woman': 'Julia Roberts',
    'Postcards from the Edge': 'Meryl Streep',
    'Mr. and Mrs. Bridge': 'Joanne Woodward',
    'The Silence of the Lambs': 'Jodie Foster',
    'Thelma & Louise': 'Geena Davis',
    'Rambling Rose': 'Laura Dern',
    'For the Boys': 'Bette Midler',
    'Thelma & Louise_': 'Susan Sarandon',
    'Howards End': 'Emma Thompson',
    'Indochine': 'Catherine Deneuve',
    'Passion Fish': 'Mary McDonnell',
    'Love Field': 'Michelle Pfeiffer',
    'Lorenzo\'s Oil': 'Susan Sarandon',
    'The Piano': 'Holly Hunter',
    'What\'s Love Got to Do with It': 'Angela Bassett',
    'Six Degrees of Separation': 'Stockard Channing',
    'The Remains of the Day': 'Emma Thompson',
    'Shadowlands': 'Debra Winger',
    'Blue Sky': 'Jessica Lange',
    'Nell': 'Jodie Foster',
    'Tom & Viv': 'Miranda Richardson',
    'Little Women': 'Winona Ryder',
    'The Client': 'Susan Sarandon',
    'Dead Man Walking': 'Susan Sarandon',
    'Leaving Las Vegas': 'Elisabeth Shue',
    'Casino': 'Sharon Stone',
    'The Bridges of Madison County': 'Meryl Streep',
    'Sense and Sensibility': 'Emma Thompson',
    'Fargo': 'Frances McDormand',
    'Secrets & Lies': 'Brenda Blethyn',
    'Marvin\'s Room': 'Diane Keaton',
    'The English Patient': 'Kristin Scott Thomas',
    'Breaking the Waves': 'Emily Watson',
    'As Good as It Gets': 'Helen Hunt',
    'The Wings of the Dove': 'Helena Bonham Carter',
    'Afterglow': 'Julie Christie',
    'Mrs. Brown': 'Judi Dench',
    'Titanic': 'Kate Winslet',
    'Shakespeare in Love': 'Gwyneth Paltrow',
    'Elizabeth': 'Cate Blanchett',
    'Central Station': 'Fernanda Montenegro',
    'One True Thing': 'Meryl Streep',
    'Hilary and Jackie': 'Emily Watson',
    'Boys Don\'t Cry': 'Hilary Swank',
    'American Beauty': 'Annette Bening',
    'Tumbleweeds': 'Janet McTeer',
    'The End of the Affair': 'Julianne Moore',
    'Music of the Heart': 'Meryl Streep',
    'Erin Brockovich': 'Julia Roberts',
    'The Contender': 'Joan Allen',
    'Chocolat': 'Juliette Binoche',
    'Requiem for a Dream': 'Ellen Burstyn',
    'You Can Count on Me': 'Laura Linney',
    'Monster\'s Ball': 'Halle Berry',
    'Iris': 'Judi Dench',
    'Moulin Rouge!': 'Nicole Kidman',
    'In the Bedroom': 'Sissy Spacek',
    'Bridget Jones\'s Diary': 'Renée Zellweger',
    'The Hours': 'Nicole Kidman',
    'Frida': 'Salma Hayek',
    'Unfaithful': 'Diane Lane',
    'Far from Heaven': 'Julianne Moore',
    'Chicago': 'Renée Zellweger',
    'Monster': 'Charlize Theron',
    'Whale Rider': 'Keisha Castle-Hughes',
    'Something\'s Gotta Give': 'Diane Keaton',
    'In America': 'Samantha Morton',
    '21 Grams': 'Naomi Watts',
    'Million Dollar Baby': 'Hilary Swank',
    'Being Julia': 'Annette Bening',
    'Maria Full of Grace': 'Catalina Sandino Moreno',
    'Vera Drake': 'Imelda Staunton',
    'Eternal Sunshine of the Spotless Mind': 'Kate Winslet',
    'Walk the Line': 'Reese Witherspoon',
    'Mrs Henderson Presents': 'Judi Dench',
    'Transamerica': 'Felicity Huffman',
    'Pride & Prejudice': 'Keira Knightley',
    'North Country': 'Charlize Theron',
    'The Queen': 'Helen Mirren',
    'Volver': 'Penélope Cruz',
    'Notes on a Scandal': 'Judi Dench',
    'The Devil Wears Prada': 'Meryl Streep',
    'Little Children': 'Kate Winslet',
    'La Vie en Rose': 'Marion Cotillard',
    'Elizabeth: The Golden Age': 'Cate Blanchett',
    'Away from Her': 'Julie Christie',
    'The Savages': 'Laura Linney',
    'Juno': 'Ellen Page',
    'The Reader': 'Kate Winslet',
    'Rachel Getting Married': 'Anne Hathaway',
    'Changeling': 'Angelina Jolie',
    'Frozen River': 'Melissa Leo',
    'Doubt': 'Meryl Streep',
    'The Blind Side': 'Sandra Bullock',
    'The Last Station': 'Helen Mirren',
    'An Education': 'Carey Mulligan',
    'Precious': 'Gabourey Sidibe',
    'Julie & Julia': 'Meryl Streep',
}



from foba.tabulars.tables.AeroEngineSpecs import AeroEngineSpecs
from foba.tabulars.tables.AoEIIUnits import AoEIIUnits
from foba.tabulars.tables.BigMacAdjustedIndexes import BigMacAdjustedIndexes
from foba.tabulars.tables.BistroDutyRoster import BistroDutyRoster
from foba.tabulars.tables.ChinaConceptFirms import ChinaConceptFirms
from foba.tabulars.tables.FrontierEconomies import FrontierEconomies
from foba.tabulars.tables.TopBoxOffice import TopBoxOffice
from foba.tabulars.tables.USTechFirms import USTechFirms

table_collection = {
    'AeroEngineSpecs': AeroEngineSpecs,
    'AoEIIUnits': AoEIIUnits,
    'BigMacAdjustedIndexes': BigMacAdjustedIndexes,
    'BistroDutyRoster': BistroDutyRoster,
    'ChinaConceptFirms': ChinaConceptFirms,
    'FrontierEconomies': FrontierEconomies,
    'TopBoxOffice': TopBoxOffice,
    'USTechFirms': USTechFirms,
}



from texting import DOT, SP


def brief_name(name):
    vec = name.split(SP)
    last = vec.pop()
    while last == 'Jr.':
        last = vec.pop() + last
    return DOT.join([x[0] for x in vec]) + DOT + last



from foba.matrices.matrix_numbers.functions.identity_matrix import identity_matrix
from foba.matrices.matrix_numbers.functions.pascal_matrix import \
    lower_pascal_matrix, \
    symmetric_pascal_matrix, \
    upper_pascal_matrix
from foba.matrices.matrix_numbers.functions.zig_zag_matrix import zig_zag_matrix
from foba.utils import FooDict

matrix_collection = FooDict({
    'identity_matrix': identity_matrix,
    'upper_pascal_matrix': upper_pascal_matrix,
    'lower_pascal_matrix': lower_pascal_matrix,
    'symmetric_pascal_matrix': symmetric_pascal_matrix,
    'zig_zag_matrix': zig_zag_matrix,
})



softwoods = {
    'DouglasFir': 'joinery, doors and heavy construction',
    'EasternWhitePine': 'furniture',
    'EuropeanSpruce': 'used throughout construction, panelling and cladding',
    'Larch': 'used for cladding and boats',
    'LodgepolePine': 'roofing, flooring and in making chipboard and particle board[4]',
    'ParanaPine': 'stair treads and joinery',
    'RedCeda': 'furniture, decking, cladding, and roof shingles',
    'ScotsPine': 'construction industry, mostly for interior work',
    'SitkaSpruce': 'musical instrument, airplane',
    'SouthernYellowPine': 'joinery, flooring and decking',
    'WesternHemlock': 'doors, joinery and furniture',
    'Yew': 'interior and exterior furniture e.g. chairs, gate posts and wood turning',
}



movie_quotes = dict([
  ('Love Story', "Love means never having to say you're sorry.",),
  ('Braveheart', "They may take our lives, but they'll never take our freedom!",),
  ('In the Heat of the Night', "They call me Mister Tibbs!",),
  ('When Harry Met Sally', "When you realize you want to spend the rest of your life with somebody, you want the rest of your life to start as soon as possible.",),
  ('Taken', "If you let my daughter go now, that'll be the end of it. I will not look for you, I will not pursue you. But if you don't, I will look for you, I will find you, and I will kill you.",),
  ('Jerry Maguire', "You complete me.",),
  ('Gladiator', "My name is Maximus Decimus Meridius, commander of the Armies of the North, General of the Felix Legions and loyal servant to the true emperor, Marcus Aurelius. Father to a murdered son, husband to a murdered wife. And I will have my vengeance, in this life or the next.",),
  ('There Will Be Blood', "I drink your milkshake!",),
  ('Planet of the Apes', "Get your stinking paws off me, you damned dirty ape!",),
  ('As Good as It Gets', "You make me want to be a better man.",),
  ('Clueless', "As if!",),
  ('Star Wars Episode VII: The Force Awakens', "Chewie, we're home.",),
  ('Chinatown', "Forget it, Jake. It's Chinatown.",),
  ('This Is Spinal Tap', "These go to eleven.",),
  ('Midnight Cowboy', "I'm walking here! I'm walking here!",),
  ('King Kong', "It was Beauty killed the Beast.",),
  ('The Treasure of the Sierra Madre', "Badges? We ain't got no badges! We don't need no badges! I don't have to show you any stinking badges!",),
  ('The Devil Wears Prada', "I'm just one stomach flu away from my goal weight.",),
  ('Pulp Fiction', "They call it a Royale with cheese.",),
  ('Poltergeist', "They're here!",),
  ('Snow White and the Seven Dwarves', "Magic Mirror on the wall, who is the fairest one of all?",),
  ('The Godfather: Part III', "Just when I thought I was out, they pull me back in.",),
  ('Some Like It Hot', "Nobody's perfect.",),
  ('Rocky', "Yo, Adrian!",),
  ('The Karate Kid', "Wax on, wax off.",),
  ('The Jazz Singer', "You ain't heard nothin' yet!",),
  ('Dr. Strangelove', "Gentlemen, you can't fight in here! This is the war room!",),
  ('12 Years a Slave', "I don't want to survive. I want to live.",),
  ('The Adventures of Sherlock Holmes', "Elementary, my dear Watson.",),
  ('Babe', "That'll do, pig. That'll do.",),
  ('Brokeback Mountain', "I wish I knew how to quit you.",),
  ('Good Morning', "Good morning, Vietnam!",),
  ('The Lord of the Rings: The Two Towers', "My precious.",),
  ('Argo', "Argo f— yourself.",),
  ('Frankenstein', "It's alive! It's alive!",),
  ('A Streetcar Named Desire', "I have always depended on the kindness of strangers.",),
  ('Sudden Impact', "Go ahead, make my day.",),
  ('Goodfellas', "I mean, funny like I'm a clown? I amuse you?",),
  ('Star Wars', "Help me, Obi-Wan Kenobi. You're my only hope.",),
  ('To Have and Have Not', "You know how to whistle, don't you, Steve? You just put your lips together and blow.",),
  ('The Help', "You is kind. You is smart. You is important.",),
  ('Gone With the Wind', "After all, tomorrow is another day!",),
  ('A Streetcar Named Desire', "Stella! Hey, Stella!",),
  ('The Wizard of Oz', "Pay no attention to that man behind the curtain!",),
  ('Notting Hill', "I'm also just a girl, standing in front of a boy, asking him to love her.",),
  ('The Big Lebowski', "The Dude abides.",),
  ('Terminator 2: Judgment Day', "Hasta la vista, baby.",),
  ('The Wizard of Oz', "I'll get you, my pretty, and your little dog, too!",),
  ('Casablanca', "Play it, Sam. Play 'As Time Goes By.'",),
  ('The Silence of the Lambs', "I'm having an old friend for dinner.",),
  ('Who Framed Roger Rabbit', "I'm not bad. I'm just drawn that way.",),
  ('Field of Dreams', "If you build it, he will come.",),
  ('Finding Nemo', "Just keep swimming.",),
  ('Forrest Gump', "Mama says, 'Stupid is as stupid does.'",),
  ('Titanic', "I'm the king of the world!",),
  ('Goldfinger', "Shaken, not stirred.",),
  ('Cool Hand Luke', "What we've got here is a failure to communicate.",),
  ('Sunset Boulevard', "I am big! It's the pictures that got small.",),
  ('It\'s a Wonderful Life', "Every time a bell rings, an angel gets his wings.",),
  ('The Godfather', "Keep your friends close, but your enemies closer.",),
  ('The Usual Suspects', "The greatest trick the devil ever pulled was convincing the world he didn't exist.",),
  ('Network', "I'm as mad as hell, and I'm not going to take this anymore!",),
  ('On the Waterfront', "You don't understand! I could've had class. I could've been a contender. I could've been somebody, instead of a bum, which is what I am.",),
  ('Back to the Future', "Roads? Where we're going we don't need roads.",),
  ('All About Eve', "Fasten your seatbelts. It's going to be a bumpy night.",),
  ('Apocalypse Now', "I love the smell of napalm in the morning.",),
  ('Dirty Harry', "You've got to ask yourself one question: 'Do I feel lucky?' Well, do ya punk?",),
  ('Scarface', "Say hello to my little friend!",),
  ('Jerry Maguire', "Show me the money!",),
  ('The Godfather', "Leave the gun. Take the cannoli.",),
  ('Dead Poets Society', "Carpe diem. Seize the day, boys.",),
  ('The Graduate', "Mrs. Robinson, you're trying to seduce me, aren't you?",),
  ('Airplane', "I am serious. And don't call me Shirley.",),
  ('The Shining', "Here's Johnny!",),
  ('A League of Their Own', "There's no crying in baseball!",),
  ('Jerry Maguire', "You had me at hello.",),
  ('Apollo 13', "Houston, we have a problem.",),
  ('Toy Story', "To infinity and beyond!",),
  ('Die Hard', "Yippie-ki-yay, motherf—er!",),
  ('E.T. the Extra-Terrestrial', "E.T. phone home.",),
  ('A Few Good Men', "You can't handle the truth!",),
  ('The Terminator', "I'll be back.",),
  ('The Sixth Sense', "I see dead people.",),
  ('Dr. No', "Bond. James Bond.",),
  ('Casablanca', "We'll always have Paris.",),
  ('Casablanca', "This is the beginning of a beautiful friendship.",),
  ('When Harry Met Sally', "I'll have what she's having.",),
  ('The Dark Knight', "Why so serious?",),
  ('The Princess Bride', "Hello. My name is Inigo Montoya. You killed my father. Prepare to die.",),
  ('Star Wars Episode V: The Empire Strikes Back', "I am your father.",),
  ('Fight Club', "The first rule of Fight Club is: You do not talk about Fight Club.",),
  ('The Wizard of Oz', "There's no place like home.",),
  ('Taxi Driver', "You talkin' to me?",),
  ('Casablanca', "Of all the gin joints in all the towns in all the world, she walks into mine.",),
  ('The Godfather', "I'm going to make him an offer he can't refuse.",),
  ('The Wizard of Oz', "Toto, I've a feeling we're not in Kansas anymore.",),
  ('Star Wars', "May the Force be with you.",),
  ('Jaws', "You're gonna need a bigger boat.",),
  ('Casablanca', "Here's looking at you, kid.",),
  ('Gone With the Wind', "Frankly, my dear, I don't give a damn.",),
])





actors = [
    'Jeremy',
    'Kevin',
    'Robert',
    'Gérard',
    'Richard',
    'Anthony',
    'Warren',
    'Robert',
    'Nick',
    'Robin',
    'Al',
    'Robert',
    'Clint',
    'Stephen',
    'Denzel',
    'Tom',
    'Daniel',
    'Laurence',
    'Anthony',
    'Liam',
    'Tom',
    'Morgan',
    'Nigel',
    'Paul',
    'John',
    'Nicolas',
    'Richard',
    'Anthony',
    'Sean',
    'Massimo',
    'Geoffrey',
    'Tom',
    'Ralph',
    'Woody',
    'Billy',
    'Jack',
    'Matt',
    'Robert',
    'Peter',
    'Dustin',
    'Roberto',
    'Tom',
    'Ian',
    'Nick',
    'Edward',
    'Kevin',
    'Russell',
    'Richard',
    'Sean',
    'Denzel',
    'Russell',
    'Javier',
    'Tom',
    'Ed',
    'Geoffrey',
    'Denzel',
    'Russell',
    'Sean',
    'Will',
    'Tom',
    'Adrien',
    'Nicolas',
    'Michael',
    'Daniel',
    'Jack',
    'Sean',
    'Johnny',
    'Ben',
    'Jude',
    'Bill',
    'Jamie',
    'Don',
    'Johnny',
    'Leonardo',
    'Clint',
    'Philip',
    'Terrence',
    'Heath',
    'Joaquin',
    'David',
    'Forest',
    'Leonardo',
    'Ryan',
    'Peter',
    'Will',
    'Daniel',
    'George',
    'Johnny',
    'Tommy',
    'Viggo',
    'Sean',
    'Richard',
    'Frank',
    'Brad',
    'Mickey',
    'Jeff',
    'George',
    'Colin',
    'Morgan',
    'Jeremy',
]



mega_cities = {
    'PearlRiverDelta': 46.7,
    'Tokyo': 40.4,
    'Shanghai': 33.6,
    'Jakarta': 31.3,
    'Delhi': 30.3,
    'Manila': 25.7,
    'Bombay': 25.1,
    'Seoul': 24.8,
    'MexicoCity': 23.0,
    'SãoPaulo': 22.4,
    'NewYork': 22.1,
    'Cairo': 21.0,
    'Dacca': 20.2,
    'Beijing': 19.8,
    'Lagos': 19.4,
    'Bangkok': 18.8,
    'Karachi': 17.8,
    'LosAngeles': 17.7,
    'Osaka': 17.7,
    'Moscow': 17.3,
    'Calcutta': 16.8,
    'BuenosAires': 16.4,
    'Istanbul': 16.0,
    'Tehran': 15.3,
    'London': 14.8,
    'Johannesburg': 13.9,
    'RiodeJaneiro': 13.2,
    'Lahore': 13.0,
    'Tientsin': 12.7,
    'Kinshasa': 12.4,
    'Bangalore': 12.2,
    'Paris': 11.4,
    'Madras': 11.3,
    'Nagoya': 10.5,
    'HyderabadIndia': 10.2,
    'Lima': 10.1,
    'Xiamen': 10.0,
    'Chicago': 9.8,
    'Bogotá': 9.6,
    'Chengtu': 9.6,
    'Taipei': 9.2,
    'Wuhan': 9.1,
    'KualaLumpur': 8.9,
    'Saigon': 8.6,
    'Washington': 8.6,
    'Ahmedabad': 8.5,
    'Hangchou': 8.5,
    'Chungking': 8.2,
    'Luanda': 8.2,
    'Riyadh': 8.1,
    'Shenyang': 8.0,
    'Singapore': 7.9,
    'SanFrancisco': 7.9,
    'Boston': 7.8,
    'Shantou': 7.7,
    'HongKong': 7.5,
    'Toronto': 7.5,
    'Philadelphia': 7.4,
    'Santiago': 7.3,
    'Dallas': 7.2,
    'Baghdad': 7.0,
    'Bandung': 7.0,
    'Sian': 7.0,
    'Poona': 6.9,
    'Houston': 6.8,
    'Nanking': 6.6,
    'Surat': 6.6,
    'Madrid': 6.6,
    'DaresSalaam': 6.5,
    'Miami': 6.4,
    'Khartoum': 6.3,
    'Milan': 6.2,
    'Tsingtao': 6.1,
    'Surabaya': 6.1,
    'Nairobi': 5.9,
    'Atlanta': 5.9,
    'Alexandria': 5.8,
    'Detroit': 5.8,
    'TheRuhr': 5.7,
    'SaintPetersburg': 5.7,
    'Abidjan': 5.6,
    'Amman': 5.6,
    'Rangoon': 5.6,
    'Dubai': 5.4,
    'Guadalajara': 5.4,
    'Wenzhou': 5.4,
    'Sydney': 5.3,
    'Accra': 5.1,
    'Harbin': 5.1,
    'Ankara': 5.1,
    'BeloHorizonte': 5.0,
    'CologneDusseldorf': 5.0,
    'Monterrey': 5.0,
    'MelbourneAus': 5.0,
    'Hofei': 5.0,
    'Jeddah': 5.0,
    'Zhengzhou': 5.0,
    'Changsha': 4.9,
    'Chittagong': 4.9,
    'Barcelona': 4.8,
    'KuwaitCity': 4.8,
    'Colombo': 4.8,
    'Berlin': 4.7,
    'Kano': 4.7,
    'Phoenix': 4.7,
    'Shijiazhuang': 4.6,
    'Dairen': 4.6,
    'Casablanca': 4.5,
    'Jinan': 4.5,
    'Seattle': 4.4,
    'Tampa': 4.4,
    'Kabul': 4.3,
    'Kunming': 4.3,
    'CapeTown': 4.2,
    'Montreal': 4.2,
    'Naples': 4.2,
    'Pusan': 4.1,
    'PortoAlegre': 4.1,
    'Taiyuan': 4.1,
    'Ürümqi': 4.1,
    'Fuzhou': 4.0,
    'Medellín': 4.0,
    'Brasília': 4.0,
    'Medan': 4.0,
    'SantoDomingo': 4.0,
    'Algiers': 4.0,
    'Jaipur': 4.0,
    'Nanchang': 3.9,
    'Recife': 3.9,
    'Damascus': 3.8,
    'AddisAbaba': 3.8,
    'Hanoi': 3.8,
    'Caracas': 3.8,
    'Denver': 3.8,
    'Dakar': 3.7,
    'Lucknow': 3.7,
    'Changchun': 3.7,
    'Fortaleza': 3.7,
    'Faisalabad': 3.7,
    'Salvador': 3.6,
    'Ibadan': 3.6,
    'Rome': 3.6,
    'Abuja': 3.5,
    'Bamako': 3.5,
    'Kampala': 3.5,
    'Zibo': 3.5,
    'Kanpur': 3.5,
    'Kiev': 3.5,
    'Nanning': 3.5,
    'Orlando': 3.5,
    'Rawalpindi': 3.5,
    'Yaoundé': 3.5,
    'Curitiba': 3.4,
    'Durban': 3.4,
    'Douala': 3.4,
    'Athens': 3.4,
    'SanDiego': 3.4,
    'Guiyang': 3.3,
    'Nagpur': 3.3,
    'Ningbo': 3.3,
    'FrankfurtamMain': 3.3,
    'Rotterdam': 3.3,
    'PortAuPrince': 3.2,
    'Tashkent': 3.2,
    'Campinas': 3.2,
    'Isfahan': 3.2,
    'Meshed': 3.2,
    'Indore': 3.2,
    'Minneapolis': 3.2,
    'Puebla': 3.2,
    'Birmingham': 3.2,
    'Brisbane': 3.2,
    'GuatemalaCity': 3.2,
    'Guayaquil': 3.2,
    'Kathmandu': 3.1,
    'Cleveland': 3.1,
    'Manchester': 3.1,
    'Izmir': 3.0,
    'Kumasi': 3.0,
    'Pyongyang': 3.0,
    'Bhilai': 2.9,
    'Lanzhou': 2.9,
    'Cali': 2.9,
    'Hamburg': 2.9,
    'Tangshan': 2.9,
    'Vancouver': 2.9,
    'Xuzhou': 2.9,
    'Huizhou': 2.8,
    'Sana_a': 2.8,
    'Cincinnati': 2.8,
    'Kaohsiung': 2.8,
    'Gujranwala': 2.8,
    'Taegu': 2.8,
    'Fukuoka': 2.8,
    'Cebu': 2.7,
    'Quito': 2.7,
    'Brussels': 2.7,
    'Coimbatore': 2.7,
    'TelAviv_Jaffa': 2.7,
    'Lusaka': 2.7,
    'Patna': 2.7,
    'Tunis': 2.7,
    'Anshan': 2.6,
    'Budapest': 2.6,
    'Maracaibo': 2.6,
    'Ouagadougou': 2.6,
    'Baku': 2.6,
    'Luoyang': 2.6,
    'Conakry': 2.5,
    'Dammam': 2.5,
    'Sapporo': 2.5,
    'Amsterdam': 2.5,
    'Charlotte': 2.5,
    'Jiangyin': 2.5,
    'Lisbon': 2.5,
    'SaltLakeCity': 2.5,
    'Doha': 2.4,
    'Bhopal': 2.4,
    'Goiânia': 2.4,
    'Portland': 2.4,
    'Taichung': 2.4,
    'Stuttgart': 2.4,
    'SanJosé': 2.4,
    'Warsaw': 2.4,
    'Asunción': 2.4,
    'Chandigarh': 2.4,
    'Katowice': 2.4,
    'Peshawar': 2.4,
    'PortHarcourt': 2.4,
    'StLouis': 2.4,
    'Toluca': 2.3,
    'Agra': 2.3,
    'Taizhou': 2.3,
    'HyderabadPakistan': 2.3,
    'LasVegas': 2.3,
    'Maputo': 2.3,
    'Brazzaville': 2.3,
    'Munich': 2.3,
    'Semarang': 2.3,
    'Vienna': 2.3,
    'Baotou': 2.2,
    'Belém': 2.2,
    'Havana': 2.2,
    'SanAntonio': 2.2,
    'Stockholm': 2.2,
    'Vadodara': 2.2,
    'Bucharest': 2.2,
    'Lubumbashi': 2.2,
    'Manaus': 2.2,
    'Okayama': 2.2,
    'Vishakhapatnam': 2.2,
    'Yantai': 2.2,
    'Leeds': 2.2,
    'SantaCruz': 2.2,
    'GeorgeTown': 2.1,
    'LaPaz': 2.1,
    'Mecca': 2.1,
    'Xinxiang': 2.1,
    'Alma_Ata': 2.1,
    'Barranquilla': 2.1,
    'Lomé': 2.1,
    'Multan': 2.1,
    'Baoding': 2.1,
    'Bursa': 2.1,
    'Mbuji_Mayi': 2.1,
    'Nasik': 2.1,
    'Perth': 2.1,
    'PhnomPenh': 2.1,
    'Sacramento': 2.1,
    'Harare': 2.1,
    'Linyi': 2.1,
    'Pittsburgh': 2.1,
    'Rabat': 2.1,
    'Cixi': 2.0,
    'Indianapolis': 2.0,
    'Kitakyushu': 2.0,
    'Liuzhou': 2.0,
    'Minsk': 2.0,
    'ValenciaVenuezuela': 2.0,
    'Vijayawada': 2.0,
    'KansasCity': 2.0,
    'Nantong': 2.0,
    'Weifang': 2.0,
    'Huai_an': 2.0,
    'Ludhiana': 2.0,
    'Lyon': 2.0,
    'Tijuana': 2.0,
    'Austin': 2.0,
    'Gaza': 2.0,
    'Yangzhou': 2.0,
    'León': 1.9,
    'Mogadishu': 1.9,
    'Benares': 1.9,
    'Yogyakarta': 1.9,
    'Huainan': 1.9,
    'SanJuan': 1.9,
    'Aleppo': 1.9,
    'Bhubaneswar': 1.9,
    'SanSalvador': 1.9,
    'Makassar': 1.9,
    'Rajkot': 1.9,
    'Hiroshima': 1.9,
    'Kaduna': 1.9,
    'Liverpool': 1.9,
    'Xiangyang': 1.9,
    'Hohhot': 1.9,
    'Haikou': 1.9,
    'Montevideo': 1.9,
    'Vitória': 1.9,
    'ValenciaSpain': 1.8,
    'Cotonou': 1.8,
    'Tabriz': 1.8,
    'Aurangabad': 1.8,
    'Freetown': 1.8,
    'Madurai': 1.8,
    'Shiraz': 1.8,
    'Columbus': 1.8,
    'Beirut': 1.8,
    'Malang': 1.8,
    'Zhuhai': 1.8,
    'Adana': 1.7,
    'Meerut': 1.7,
    'Santos': 1.7,
    'NizhnyNovgorod': 1.7,
    'Palembang': 1.7,
    'Anyang': 1.7,
    'Hartford': 1.7,
    'Hengyang': 1.7,
    'Novosibirsk': 1.7,
    'Glasgow': 1.7,
    'Copenhagen': 1.7,
    'Córdoba': 1.7,
    'Gaziantep': 1.7,
    'Handan': 1.7,
    'Jamshedpur': 1.7,
    'Yekaterinburg': 1.7,
    'Marseille': 1.7,
    'Raleigh': 1.7,
    'Yiwu': 1.7,
    'BeninCity': 1.7,
    'Datong': 1.7,
    'Kharkiv': 1.6,
    'Turin': 1.6,
    'Surakarta': 1.6,
    'Daqing': 1.6,
    'Āsansol': 1.6,
    'VirginiaBeach': 1.6,
    'Kwangju': 1.6,
    'Srinagar': 1.6,
    'Cochin': 1.6,
    'Monrovia': 1.6,
    'Mannheim': 1.6,
    'Davao': 1.6,
    'Sheffield': 1.6,
    'Guilin': 1.6,
    'Mandalay': 1.6,
    'Denpasar': 1.6,
    'Milwaukee': 1.6,
    'Medina': 1.5,
    'Bengbu': 1.5,
    'PanamaCity': 1.5,
    'Taejon': 1.5,
    'Jilin': 1.5,
    'Porto': 1.5,
    'Manama': 1.5,
    'Basra': 1.5,
    'Huaibei': 1.5,
    'Xingtai': 1.5,
    'AbuDhabi': 1.5,
    'Calgary': 1.5,
    'CiudadJuárez': 1.5,
    'Ganzhou': 1.5,
    'Sendai': 1.5,
    'Yerevan': 1.5,
    'Angeles': 1.5,
    'Jabalpur': 1.5,
    'Jodhpur': 1.5,
    'Nashville': 1.5,
    'Newcastle': 1.5,
    'Oran': 1.5,
    'Wuhu': 1.5,
    'Zhangjiakou': 1.5,
    'Jiaozuo': 1.5,
    'Yinchuan': 1.5,
    'Allahabad': 1.5,
    'Xining': 1.5,
    'Jacksonville': 1.5,
    'Tai_an': 1.5,
    'Donetsk': 1.4,
    'Prague': 1.4,
    'Querétaro': 1.4,
    'SãoLuís': 1.4,
    'Muscat': 1.4,
    'Ranchi': 1.4,
    'Zurich': 1.4,
    'Belgrade': 1.4,
    'Maoming': 1.4,
    'Maracay': 1.4,
    'Yancheng': 1.4,
    'Auckland': 1.4,
    'Chelyabinsk': 1.4,
    'Cochabamba': 1.4,
    'Natal': 1.4,
    'Ndjamena': 1.4,
    'Volgograd': 1.4,
    'Mosul': 1.4,
    'Amritsar': 1.4,
    'Gwalior': 1.4,
    'Tirupur': 1.4,
    'UlanBator': 1.4,
    'Xiangtan': 1.4,
    'Ashgabat': 1.4,
    'Batam': 1.4,
    'Dnepropetrovsk': 1.4,
    'Dublin': 1.4,
    'Ahvaz': 1.4,
    'Dhanbad': 1.4,
    'Nottingham': 1.4,
    'Putian': 1.4,
    'Yichang': 1.4,
    'Edmonton': 1.4,
    'Kotā': 1.4,
    'Rosario': 1.4,
    'Torreón': 1.4,
    'Buffalo': 1.4,
    'Chonburi': 1.4,
    'PortElizabeth': 1.4,
    'Tbilisi': 1.4,
    'Adelaide': 1.3,
    'Tananarive': 1.3,
    'Harrisburg': 1.3,
    'Qinhuangdao': 1.3,
    'Zhuzhou': 1.3,
    'Managua': 1.3,
    'Bareilly': 1.3,
    'Hamamatsu': 1.3,
    'Tainan': 1.3,
    'Zhanjiang': 1.3,
    'Antalya': 1.3,
    'Barquisimeto': 1.3,
    'Helsinki': 1.3,
    'Konya': 1.3,
    'McAllen': 1.3,
    'Niamey': 1.3,
    'Rostov_on_Don': 1.3,
    'Samara': 1.3,
    'Sofia': 1.3,
    'Lille': 1.3,
    'Qom': 1.3,
    'Sevilla': 1.3,
    'Fes': 1.3,
    'Nouakchott': 1.3,
    'Bucaramanga': 1.3,
    'Kazan': 1.3,
    'Morādābād': 1.3,
    'Nanyang': 1.3,
    'Stockton': 1.3,
    'Benguela': 1.3,
    'Ottawa': 1.3,
    'Tripoli': 1.3,
    'Zunyi': 1.3,
    'Alīgarh': 1.2,
    'Jining': 1.2,
    'Kananga': 1.2,
    'Mianyang': 1.2,
    'Mysore': 1.2,
    'Rizhao': 1.2,
    'SanLuisPotosí': 1.2,
    'Antwerp': 1.2,
    'Mombasa': 1.2,
    'Nanchong': 1.2,
    'Pekanbaru': 1.2,
    'Fuyang': 1.2,
    'Jingzhou': 1.2,
    'Maiduguri': 1.2,
    'Naha': 1.2,
    'SanPedroSula': 1.2,
    'Tiruchirappalli': 1.2,
    'BandarLampung': 1.2,
    'Baoji': 1.2,
    'Karawang': 1.2,
    'MelbourneUS': 1.2,
    'Nuremberg': 1.2,
    'Onitsha': 1.2,
    'Weihai': 1.2,
    'Agadir': 1.2,
    'Bangui': 1.2,
    'Oslo': 1.2,
    'Kigali': 1.2,
    'Luzhou': 1.2,
    'Ma_anshan': 1.2,
    'Pointe_Noire': 1.2,
    'Aguascalientes': 1.2,
    'Changde': 1.2,
    'Kisangani': 1.2,
    'Memphis': 1.2,
    'Southampton': 1.2,
    'Aba': 1.2,
    'Hanover': 1.2,
    'Kumamoto': 1.2,
    'Lianyungang': 1.2,
    'Mérida': 1.2,
    'Serang': 1.2,
    'Tegucigalpa': 1.2,
    'Yueyang': 1.2,
    'Zhenjiang': 1.2,
    'Gauhati': 1.1,
    'Hubli_Dharwar': 1.1,
    'OklahomaCity': 1.1,
    'Quetta': 1.1,
    'Shizuoka': 1.1,
    'Sousse': 1.1,
    'Wanzhou': 1.1,
    'Cartagena': 1.1,
    'Ulanhad': 1.1,
    'Greensboro': 1.1,
    'JoãoPessoa': 1.1,
    'Krasnoyarsk': 1.1,
    'Suzhou': 1.1,
    'Ufa': 1.1,
    'Cirebon': 1.1,
    'Jullundur': 1.1,
    'Kaifeng': 1.1,
    'Marrakesh': 1.1,
    'Nur_Sultan': 1.1,
    'Omsk': 1.1,
    'Cangnan': 1.1,
    'Maceió': 1.1,
    'Odessa': 1.1,
    'Perm': 1.1,
    'Salem': 1.1,
    'Tangier': 1.1,
    'Ulsan': 1.1,
    'Benxi': 1.1,
    'Jinzhou': 1.1,
    'Khulna': 1.1,
    'Erbil': 1.1,
    'Bishkek': 1.1,
    'CapeCoral': 1.1,
    'Kayseri': 1.1,
    'Mendoza': 1.1,
    'Saratov': 1.1,
    'Sholapur': 1.1,
    'Arequipa': 1.1,
    'Bordeaux': 1.1,
    'Touba': 1.1,
    'Ilorin': 1.1,
    'Qingyuan': 1.1,
    'Richmond': 1.1,
    'Lilongwe': 1.1,
    'Louisville': 1.1,
    'Mexicali': 1.1,
    'Tengzhou': 1.1,
    'Voronezh': 1.1,
    'Zaozhuang': 1.1,
    'Changshu': 1.1,
    'Cuernavaca': 1.1,
    'Qiqihar': 1.1,
    'Shymkent': 1.1,
    'Suqian': 1.1,
    'Changwon': 1.0,
    'Changzhi': 1.0,
    'Diyarbakır': 1.0,
    'Pingxiang': 1.0,
    'Teresina': 1.0,
    'Toulouse': 1.0,
    'ElPaso': 1.0,
    'Kirkuk': 1.0,
    'Málaga': 1.0,
    'Panjin': 1.0,
    'Yibin': 1.0,
    'Zigong': 1.0,
    'Jos': 1.0,
    'Kermānshāh': 1.0,
    'Calicut': 1.0,
    'Pingdingshan': 1.0,
    'SanMigueldeTucumán': 1.0,
    'Shiliguri': 1.0,
    'Valparaíso': 1.0,
    'DaNang': 1.0,
    'Dushanbe': 1.0,
    'Florence': 1.0,
    'NewOrleans': 1.0,
    'Yingkou': 1.0,
    'Banjul': 1.0,
    'Bremen': 1.0,
    'Enugu': 1.0,
    'Florianópolis': 1.0,
    'Hechuan': 1.0,
    'Honolulu': 1.0,
    'Jiaxing': 1.0,
    'Mersin': 1.0,
    'Morelia': 1.0,
    'Tucson': 1.0,
    'Warri': 1.0,
    'Zaria': 1.0,
}



AoEIIUnitsAttackByStages = {
    'side': ['Imperial', 'Castle', 'Feudal', 'Dark'],
    'head': ['Barracks', 'Archery Range', 'Stable', 'Siege Workshop'],
    'rows': [
        [9.8, 10, 10, 31.3],
        [6.7, 4.7, 7.3, 18],
        [4.5, 3, 5, None],
        [4, None, None, None],
    ],
}



AoEIIUnitsHpByStages = {
    'side': ['Imperial', 'Castle', 'Feudal', 'Dark'],
    'head': ['Barracks', 'Archery Range', 'Stable', 'Siege Workshop'],
    'rows': [
        [62.5, 45, 118.8, 121.7],
        [53.3, 40, 86.7, 88.3],
        [45, 17, 45, None],
        [40, None, None, None],
    ],
}



directors = [
    'Kevin',
    'Francis',
    'Stephen',
    'Barbet',
    'Martin',
    'Jonathan',
    'Barry',
    'Ridley',
    'John',
    'Oliver',
    'Clint',
    'Robert',
    'Martin',
    'James',
    'Neil',
    'Steven',
    'Robert',
    'Jane',
    'James',
    'Jim',
    'Robert',
    'Woody',
    'Krzysztof',
    'Robert',
    'Quentin',
    'Mel',
    'Mike',
    'Chris',
    'Michael',
    'Tim',
    'Anthony',
    'Joel',
    'Miloš',
    'Scott',
    'Mike',
    'James',
    'Peter',
    'Atom',
    'Curtis',
    'Gus',
    'Steven',
    'Roberto',
    'John',
    'Terrence',
    'Peter',
    'Sam',
    'Lasse',
    'Spike',
    'Michael',
    'M',
    'Steven',
    'Stephen',
    'Ang',
    'Ridley',
    'Steven',
    'Ron',
    'Robert',
    'Peter',
    'David',
    'Roman',
    'Ridley',
    'Pedro',
    'Stephen',
    'Rob',
    'Peter',
    'Martin',
    'Sofia',
    'Clint',
    'Fernando',
    'Peter',
    'Clint',
    'Taylor',
    'Mike',
    'Alexander',
    'Martin',
    'Ang',
    'George',
    'Paul',
    'Bennett',
    'Steven',
    'Martin',
    'Clint',
    'Stephen',
    'Alejandro',
    'Paul',
    'Joel',
    'Ethan',
    'Paul',
    'Tony',
    'Jason',
    'Julian',
    'Danny',
    'Stephen',
    'David',
    'Ron',
    'Gus',
    'James',
    'Kathryn',
    'Lee',
    'Jason',
    'Quentin',
]



film_directors = {
    'Dances with Wolves': 'Kevin Costner',
    'The Godfather Part III': 'Francis Ford Coppola',
    'The Grifters': 'Stephen Frears',
    'Reversal of Fortune': 'Barbet Schroeder',
    'Goodfellas': 'Martin Scorsese',
    'The Silence of the Lambs': 'Jonathan Demme',
    'Bugsy': 'Barry Levinson',
    'Thelma & Louise': 'Ridley Scott',
    'Boyz n the Hood': 'John Singleton',
    'JFK': 'Oliver Stone',
    'Unforgiven': 'Clint Eastwood',
    'The Player': 'Robert Altman',
    'Scent of a Woman': 'Martin Brest',
    'Howards End': 'James Ivory',
    'The Crying Game': 'Neil Jordan',
    'Schindler\'s List': 'Steven Spielberg',
    'Short Cuts': 'Robert Altman',
    'The Piano': 'Jane Campion',
    'The Remains of the Day': 'James Ivory',
    'In the Name of the Father': 'Jim Sheridan',
    'Forrest Gump': 'Robert Zemeckis',
    'Bullets Over Broadway': 'Woody Allen',
    'Three Colors: Red': 'Krzysztof Kieślowski',
    'Quiz Show': 'Robert Redford',
    'Pulp Fiction': 'Quentin Tarantino',
    'Braveheart': 'Mel Gibson',
    'Leaving Las Vegas': 'Mike Figgis',
    'Babe': 'Chris Noonan',
    'Il Postino': 'Michael Radford',
    'Dead Man Walking': 'Tim Robbins',
    'The English Patient': 'Anthony Minghella',
    'Fargo': 'Joel Coen',
    'The People vs. Larry Flynt': 'Miloš Forman',
    'Shine': 'Scott Hicks',
    'Secrets & Lies': 'Mike Leigh',
    'Titanic': 'James Cameron',
    'The Full Monty': 'Peter Cattaneo',
    'The Sweet Hereafter': 'Atom Egoyan',
    'L.A. Confidential': 'Curtis Hanson',
    'Good Will Hunting': 'Gus Van Sant',
    'Saving Private Ryan': 'Steven Spielberg',
    'Life Is Beautiful': 'Roberto Benigni',
    'Shakespeare in Love': 'John Madden',
    'The Thin Red Line': 'Terrence Malick',
    'The Truman Show': 'Peter Weir',
    'American Beauty': 'Sam Mendes',
    'The Cider House Rules': 'Lasse Hallström',
    'Being John Malkovich': 'Spike Jonze',
    'The Insider': 'Michael Mann',
    'The Sixth Sense': 'M. Night Shyamalan',
    'Traffic': 'Steven Soderbergh',
    'Billy Elliot': 'Stephen Daldry',
    'Crouching Tiger, Hidden Dragon': 'Ang Lee',
    'Gladiator': 'Ridley Scott',
    'Erin Brockovich': 'Steven Soderbergh',
    'A Beautiful Mind': 'Ron Howard',
    'Gosford Park': 'Robert Altman',
    'The Lord of the Rings: The Fellowship of the Ring': 'Peter Jackson',
    'Mulholland Drive': 'David Lynch',
    'The Pianist': 'Roman Polanski',
    'Black Hawk Down': 'Ridley Scott',
    'Talk to Her': 'Pedro Almodóvar',
    'The Hours': 'Stephen Daldry',
    'Chicago': 'Rob Marshall',
    'The Lord of the Rings: The Return of the King': 'Peter Jackson',
    'Gangs of New York': 'Martin Scorsese',
    'Lost in Translation': 'Sofia Coppola',
    'Mystic River': 'Clint Eastwood',
    'City of God': 'Fernando Meirelles',
    'Master and Commander: The Far Side of the World': 'Peter Weir',
    'Million Dollar Baby': 'Clint Eastwood',
    'Ray': 'Taylor Hackford',
    'Vera Drake': 'Mike Leigh',
    'Sideways': 'Alexander Payne',
    'The Aviator': 'Martin Scorsese',
    'Brokeback Mountain': 'Ang Lee',
    'Good Night, and Good Luck.': 'George Clooney',
    'Crash': 'Paul Haggis',
    'Capote': 'Bennett Miller',
    'Munich': 'Steven Spielberg',
    'The Departed': 'Martin Scorsese',
    'Letters from Iwo Jima': 'Clint Eastwood',
    'The Queen': 'Stephen Frears',
    'Babel': 'Alejandro González Iñárritu',
    'United 93': 'Paul Greengrass',
    'No Country for Old Men': 'Joel & Ethan Coen',
    'There Will Be Blood': 'Paul Thomas Anderson',
    'Michael Clayton': 'Tony Gilroy',
    'Juno': 'Jason Reitman',
    'The Diving Bell and the Butterfly': 'Julian Schnabel',
    'Slumdog Millionaire': 'Danny Boyle',
    'The Reader': 'Stephen Daldry',
    'The Curious Case of Benjamin Button': 'David Fincher',
    'Frost/Nixon': 'Ron Howard',
    'Milk': 'Gus Van Sant',
    'Avatar': 'James Cameron',
    'The Hurt Locker': 'Kathryn Bigelow',
    'Precious: Based on the Novel "Push" by Sapphire': 'Lee Daniels',
    'Up in the Air': 'Jason Reitman',
    'Inglourious Basterds': 'Quentin Tarantino',
}



film_actors = {
    'Reversal of Fortune': 'Jeremy Irons',
    'Dances with Wolves': 'Kevin Costner',
    'Awakenings': 'Robert De Niro',
    'Cyrano de Bergerac': 'Gérard Depardieu',
    'The Field': 'Richard Harris',
    'The Silence of the Lambs': 'Anthony Hopkins',
    'Bugsy': 'Warren Beatty',
    'Cape Fear': 'Robert De Niro',
    'The Prince of Tides': 'Nick Nolte',
    'The Fisher King': 'Robin Williams',
    'Scent of a Woman': 'Al Pacino',
    'Chaplin': 'Robert Downey, Jr.',
    'Unforgiven': 'Clint Eastwood',
    'The Crying Game': 'Stephen Rea',
    'Malcolm X': 'Denzel Washington',
    'Philadelphia': 'Tom Hanks',
    'In the Name of the Father': 'Daniel Day-Lewis',
    'What\'s Love Got to Do with It': 'Laurence Fishburne',
    'The Remains of the Day': 'Anthony Hopkins',
    'Schindler\'s List': 'Liam Neeson',
    'Forrest Gump': 'Tom Hanks',
    'The Shawshank Redemption': 'Morgan Freeman',
    'The Madness of King George': 'Nigel Hawthorne',
    'Nobody\'s Fool': 'Paul Newman',
    'Pulp Fiction': 'John Travolta',
    'Leaving Las Vegas': 'Nicolas Cage',
    'Mr. Holland\'s Opus': 'Richard Dreyfuss',
    'Nixon': 'Anthony Hopkins',
    'Dead Man Walking': 'Sean Penn',
    'Il Postino': 'Massimo Troisi',
    'Shine': 'Geoffrey Rush',
    'Jerry Maguire': 'Tom Cruise',
    'The English Patient': 'Ralph Fiennes',
    'The People vs. Larry Flynt': 'Woody Harrelson',
    'Sling Blade': 'Billy Bob Thornton',
    'As Good as It Gets': 'Jack Nicholson',
    'Good Will Hunting': 'Matt Damon',
    'The Apostle': 'Robert Duvall',
    'Ulee\'s Gold': 'Peter Fonda',
    'Wag the Dog': 'Dustin Hoffman',
    'Life Is Beautiful': 'Roberto Benigni',
    'Saving Private Ryan': 'Tom Hanks',
    'Gods and Monsters': 'Ian McKellen',
    'Affliction': 'Nick Nolte',
    'American History X': 'Edward Norton',
    'American Beauty': 'Kevin Spacey',
    'The Insider': 'Russell Crowe',
    'The Straight Story': 'Richard Farnsworth',
    'Sweet and Lowdown': 'Sean Penn',
    'The Hurricane': 'Denzel Washington',
    'Gladiator': 'Russell Crowe',
    'Before Night Falls': 'Javier Bardem',
    'Cast Away': 'Tom Hanks',
    'Pollock': 'Ed Harris',
    'Quills': 'Geoffrey Rush',
    'Training Day': 'Denzel Washington',
    'A Beautiful Mind': 'Russell Crowe',
    'I Am Sam': 'Sean Penn',
    'Ali': 'Will Smith',
    'In the Bedroom': 'Tom Wilkinson',
    'The Pianist': 'Adrien Brody',
    'Adaptation.': 'Nicolas Cage',
    'The Quiet American': 'Michael Caine',
    'Gangs of New York': 'Daniel Day-Lewis',
    'About Schmidt': 'Jack Nicholson',
    'Mystic River': 'Sean Penn',
    'Pirates of the Caribbean: The Curse of the Black Pearl': 'Johnny Depp',
    'House of Sand and Fog': 'Ben Kingsley',
    'Cold Mountain': 'Jude Law',
    'Lost in Translation': 'Bill Murray',
    'Ray': 'Jamie Foxx',
    'Hotel Rwanda': 'Don Cheadle',
    'Finding Neverland': 'Johnny Depp',
    'The Aviator': 'Leonardo DiCaprio',
    'Million Dollar Baby': 'Clint Eastwood',
    'Capote': 'Philip Seymour Hoffman',
    'Hustle & Flow': 'Terrence Howard',
    'Brokeback Mountain': 'Heath Ledger',
    'Walk the Line': 'Joaquin Phoenix',
    'Good Night, and Good Luck': 'David Strathairn',
    'The Last King of Scotland': 'Forest Whitaker',
    'Blood Diamond': 'Leonardo DiCaprio',
    'Half Nelson': 'Ryan Gosling',
    'Venus': 'Peter O\'Toole',
    'The Pursuit of Happyness': 'Will Smith',
    'There Will Be Blood': 'Daniel Day-Lewis',
    'Michael Clayton': 'George Clooney',
    'Sweeney Todd: The Demon Barber of Fleet Street': 'Johnny Depp',
    'In the Valley of Elah': 'Tommy Lee Jones',
    'Eastern Promises': 'Viggo Mortensen',
    'Milk': 'Sean Penn',
    'The Visitor': 'Richard Jenkins',
    'Frost/Nixon': 'Frank Langella',
    'The Curious Case of Benjamin Button': 'Brad Pitt',
    'The Wrestler': 'Mickey Rourke',
    'Crazy Heart': 'Jeff Bridges',
    'Up in the Air': 'George Clooney',
    'A Single Man': 'Colin Firth',
    'Invictus': 'Morgan Freeman',
    'The Hurt Locker': 'Jeremy Renner',
}



from types import MethodType

from foba.vectors.vector_strings.actors import actors
from foba.vectors.vector_strings.actresses import actresses
from foba.vectors.vector_strings.directors import directors
from .utils import brief_name
from ...dicts.dict_numbers import arm_sales, mega_cities, mortality_rates, power_cars, recessions
from ...dicts.dict_strings import car_plants, film_actors, film_actresses, film_directors, pastas
from ...utils import FooDict
from ...utils.flops.vector_collection import flop_shuffle

vector_collection = FooDict({
    'actors': actors,
    'actresses': actresses,
    'directors': directors,
    'carPlants': list(car_plants),
    'actorBriefs': [brief_name(x) for x in film_actors.values()],
    'actressBriefs': [brief_name(x) for x in film_actresses.values()],
    'directorBriefs': [brief_name(x) for x in film_directors.values()],
    'pastas': list(pastas),
    'armDealers': list(arm_sales),
    'megaCities': list(mega_cities),
    'deathCauses': list(mortality_rates),
    'powerCars': list(power_cars),
    'recessionYears': list(recessions),
})

vector_collection.flop_shuffle = MethodType(flop_shuffle, vector_collection)



from veho.matrix import iso


def zig_zag_matrix(n):
    mx = iso(n, n, 0)
    i, j, x = 1, 1, -1
    while ((x := x + 1) < n * n):
        mx[i - 1][j - 1] = x
        if (i + j) % 2 > 0:
            if i < n: i += 1
            else: j += 2
            if j > 1: j -= 1
        else:
            if j < n: j += 1
            else: i += 2
            if i > 1: i -= 1
    return mx



from foba.tabulars.crostabs.AoEIIUnitsAttackByStages import AoEIIUnitsAttackByStages
from foba.tabulars.crostabs.AoEIIUnitsHpByStages import AoEIIUnitsHpByStages
from foba.tabulars.crostabs.AreaByCountry import AreaByCountry
from foba.tabulars.crostabs.BigMacAdjustedIndexes import BigMacAdjustedIndexes
from foba.tabulars.crostabs.CountryGDPByYear import CountryGDPByYear
from foba.tabulars.crostabs.FinancialAssetsToGDPByYear import FinancialAssetsToGDPByYear
from foba.tabulars.crostabs.MilitaryByCountry2019 import MilitaryByCountry2019
from foba.tabulars.crostabs.TeachersCountByYear import TeachersCountByYear
from foba.utils import FooDict

crostab_collection = FooDict({
    'AoEIIUnitsAttackByStages': AoEIIUnitsAttackByStages,
    'AoEIIUnitsHpByStages': AoEIIUnitsHpByStages,
    'AreaByCountry': AreaByCountry,
    'BigMacAdjustedIndexes': BigMacAdjustedIndexes,
    'CountryGDPByYear': CountryGDPByYear,
    'FinancialAssetsToGDPByYear': FinancialAssetsToGDPByYear,
    'MilitaryByCountry2019': MilitaryByCountry2019,
    'TeachersCountByYear': TeachersCountByYear,
})



from types import MethodType

from .car_plants import car_plants
from .film_actors import film_actors
from .film_actresses import film_actresses
from .film_directors import film_directors
from .military_robots import military_robots
from .movie_quotes import movie_quotes
from .pastas import pastas
from .softwoods import softwoods
from ...utils import FooDict
from ...utils.flops.dict_collection import flop_shuffle

dict_collection = FooDict({
    'CarPlants': car_plants,
    'FilmActors': film_actors,
    'FilmActresses': film_actresses,
    'FilmDirectors': film_directors,
    'MilitaryRobots': military_robots,
    'MovieQuotes': movie_quotes,
    'Pastas': pastas,
    'Softwoods': softwoods,
})

dict_collection.flop_shuffle = MethodType(flop_shuffle, dict_collection)



from math import sqrt


class Point:

    def __init__(self, x, y, tag=None):
        self.x = x
        self.y = y
        if tag is not None: self.tag = tag

    @property
    def distance(self): return sqrt(self.x ** 2 + self.y ** 2)

    def move(self, dx, dy):
        self.x += dx
        self.y += dy
        return self

    @staticmethod
    def build(x, y): return Point(x, y)



# copyright: hbr.org
# link: https://hbr.org/2019/12/build-a-culture-to-match-your-brand

CompanyBrandTypes = [
    ['Disruptive', 'Category leader', 'Rebellious, confident, daring', ['Virgin', 'Airbnb', 'Dr Pepper'], ],
    ['Conscious', 'Higher purpose', 'Inspiring, thoughtful, transparent',
     ['Seventh', 'Generation', 'SoulCycle', 'Patagonia'], ],
    ['Service', 'Customer need', 'Humble, predictable, friendly', ['Nordstrom', 'USAA', 'Ritz Carlton'], ],
    ['Innovative', 'Possibility', 'Risk-taking, imaginative, progressive', ['Apple', 'Nike', 'Amazon'], ],
    ['Value', 'Higher-priced brand', 'Down-to-earth, practical, straightforward', ['Walmart', 'IKEA', 'Subway'], ],
    ['Performance', 'Performance standard', 'Precise, competent, reliable', ['BMW', 'FedEx', 'American Express'], ],
    ['Luxury', 'Populist brand', 'Discriminating, refined, glamorous', ['Tiffany', 'Mercedes-Benz', 'Hermes'], ],
    ['Style', 'Functional brand', 'Creative, stylish, contemporary', ['Target', 'JetBlue', 'Mini Cooper'], ],
    ['Experience', 'Customer emotion', 'Exciting, energetic, imaginative', ['Disney', 'American Girl', 'Wegmans'], ],
]



import inspect
import random


class FooDict(dict):
    def __init__(self, seq=None, **kwargs):
        super().__init__(seq, **kwargs)

    def flop_shuffle(self, length=5):
        k, o = random.choice(list(self.items()))
        if inspect.isfunction(o): return k, o(length)
        return k, o



ChinaConceptFirms = {
    'head': ['symbol', 'companyName', 'industry', 'ceo', 'sector', 'price', 'mktCap'],
    'rows': [
        ['JD', 'JD.com Inc.', 'Online Media', 'Qiangdong Richard Liu', 'Technology', 38.84, '56712614000'],
        ['BABA', 'Alibaba Group Holding Limited', 'Retail - Apparel & Specialty', 'Yong Daniel Zhang',
         'Consumer Cyclical', 205.61, '563679790000'],
        ['NTES', 'NetEase Inc.', 'Online Media', 'William Lei Ding', 'Technology', 337.91, '43239301000'],
        ['BIDU', 'Baidu Inc.', 'Online Media', 'Yanhong Robin Li', 'Technology', 124.43, '43114873000'],
        ['EDU', 'New Oriental Education', 'Education', 'Chenggang Zhou', 'Consumer Defensive', 134.47, '21303947300'],
        ['PDD', 'Pinduoduo Inc.', 'Internet Retail', None, 'Consumer Cyclical', 32.8, '38125408000'],
        ['YUMC', 'Yum China Holdings Inc.', 'Restaurants', 'Muktesh Pant', 'Consumer Cyclical', 43.81, '16468573200'],
        ['TME', 'Tencent Music Entertainment', 'Internet Content & Information', None, 'Communication Services', 12.32,
         '20760062000'],
        ['IQ', 'iQIYI, Inc.', 'Internet Content & Information', None, 'Communication Services', 23.98, '17434419200'],
        ['BILI', 'Bilibili Inc.', 'Electronic Gaming & Multimedia', None, 'Communication Services', 26.72,
         '8714006500'],
        ['LK', 'Luckin Coffee Inc.', 'Restaurants', None, 'Consumer Cyclical', 38.33, '8964160500']],
    'title': '',
}



from foba.matrices.matrix_strings.CompanyBrandTypes import CompanyBrandTypes
from foba.matrices.matrix_strings.Countries import Countries
from foba.matrices.matrix_strings.IntegratedCultureFramework import IntegratedCultureFramework
from foba.matrices.matrix_strings.MarketingMovement import MarketingMovement
from foba.utils import FooDict

matrix_collection = FooDict({
    'CompanyBrandTypes': CompanyBrandTypes,
    'Countries': Countries,
    'IntegratedCultureFramework': IntegratedCultureFramework,
    'MarketingMovement': MarketingMovement,
})



from dataclasses import dataclass
from random import randint, uniform

GOV_EXPENDITURE = 100
CONSUMPTION = 50
INVESTMENT = 100
CONSUMPTION_MARGINAL_PROPENSITY = 0.28
INVESTMENT_MULTIPLIER = 5


@dataclass
class Economy:
    g: float = GOV_EXPENDITURE
    c: float = CONSUMPTION
    i: float = INVESTMENT
    a: float = CONSUMPTION_MARGINAL_PROPENSITY
    b: float = INVESTMENT_MULTIPLIER

    @property
    def y(self):
        return self.g + self.c + self.i

    def next(self):
        k = self.c
        self.c = self.a * self.y
        self.i = self.b * (self.c - k)
        return self


def hansen_samuelson_generator(econ: Economy):
    yield econ
    while True: yield econ.next()


def hansen_samuelson(hi: int, econ: Economy = None, d: int = 2):
    if econ is None:
        cb = randint(0, 3)
        ib = randint(1, 6)
        g = int((cb + ib) / 3) * 50
        c = cb * 50
        i = ib * 50
        a = uniform(0.15, 0.50)
        b = uniform(3.0, 8.0)
        econ = Economy(g, c, i, a, b)
    return [round(econ.y, d)] + \
           [round(econ.next().y, d) for _ in range(hi - 1)]



from veho.matrix import iso


def upper_pascal_matrix(size):
    m = iso(size, size, None)
    c = -1
    while (c := c + 1) < size: m[0][c] = 1
    r = 0
    while (r := r + 1) < size:
        c = r - 1
        while (c := c + 1) < size:
            m[r][c] = m[r - 1][c - 1] + (0 if (v := m[r][c - 1]) is None else v)
    return m


def lower_pascal_matrix(size):
    m = iso(size, size, None)
    r = -1
    while (r := r + 1) < size: m[r][0] = 1
    c = 0
    while (c := c + 1) < size:
        r = c - 1
        while (r := r + 1) < size:
            m[r][c] = m[r - 1][c - 1] +(0 if (v := m[r - 1][c]) is None else v)
    return m


def symmetric_pascal_matrix(size):
    m = iso(size, size, None)
    i = -1
    while (i := i + 1) < size: m[0][i] = m[i][0] = 1
    r = 0
    while (r := r + 1) < size:
        c = 0
        while (c := c + 1) < size:
            m[r][c] = m[r - 1][c] + m[r][c - 1]
    return m



from .dict_numbers import \
    arm_sales, \
    brent_prices, \
    mega_cities, \
    mortality_rates, \
    power_cars, \
    recessions, \
    dict_collection as number_dict_collection

from .dict_strings import \
    car_plants, \
    film_actors, \
    film_actresses, \
    film_directors, \
    military_robots, \
    movie_quotes, \
    pastas, \
    softwoods, \
    dict_collection as string_dict_collection



# copyright: hbr.org
# link: https://hbr.org/2016/12/mapping-frontier-economies

FrontierEconomies = {
    'head': ['no', 'country', 'cagr', 'frontier', 'corruption', 'arbitary', 'income', 'sig drop in gdppc'],
    'rows': [
        [1, 'Bhutan', '8.47%', False, None, None, None, None],
        [2, 'Libya', '8.04%', True, True, True, None, True],
        [3, 'Myanmar', '7.88%', True, True, True, True, None],
        [4, 'Cote D\'ivore', '7.71%', True, True, None, True, None],
        [5, 'India', '7.58%', False, None, None, None, None],
        [6, 'Lao P.D.R.', '7.37%', True, True, None, None, None],
        [7, 'Mozambique', '7.16%', True, True, None, True, None],
        [8, 'Cambodia', '6.94%', True, True, None, True, None],
        [9, 'Senegal', '6.92%', True, None, None, True, None],
        [10, 'Rwanda', '6.86%', True, None, None, True, None],
        [11, 'Bangladesh', '6.84%', True, True, None, True, None],
        [12, 'Ethiopia', '6.80%', True, True, None, True, None],
        [13, 'Niger', '6.75%', True, True, None, True, None],
        [14, 'Tanzania', '6.72%', True, True, None, True, None],
        [15, 'Djibouti', '6.70%', True, True, None, None, True],
        [16, 'Kenya', '6.31%', True, True, None, True, None],
        [17, 'Panama', '6.30%', False, None, None, None, None],
        [18, 'Philippines', '6.28%', False, None, None, None, None],
        [19, 'Vietnam', '6.22%', True, True, None, None, None],
        [20, 'China', '6.14%', False, None, None, None, None],
        [21, 'Brunei', '6.11%', False, None, None, None, None],
        [22, 'Ghana', '5.94%', True, True, None, True, None],
        [23, 'Uganda', '5.85%', True, True, None, True, None],
        [24, 'Burkina Faso', '5.74%', True, True, None, True, None],
        [25, 'UzbekistanZ', '5.71%', True, True, True, None, None],
    ]
}



from math import e, pi, sqrt
from random import randint, sample

specials = {
    "PI": pi,
    "E": e,
    "SQR_2": sqrt(2),
    "SQR_3": sqrt(3),
    "SQR_5": sqrt(5),
    "GOLDEN": 1.61803398874989484820,
    "MP10_3": 10 / 3,
    "MP10_6": 10 / 6,
    "GRAVITY": 6.67430,
    "FEIGENBAUM": 4.669201609102990671853203821578,
    "GAUSS_STDEV_1": 68.2689492137,
    "GAUSS_STDEV_2": 95.4499736104,
    "GAUSS_STDEV_3": 99.7300203937,
    "LIGHT": 299792458,  # m/s**2
    "MOL": 6.02214076e+23,
    "LOSCHMIDT": 2.6867811e+25,
    "ABSOLUTE_ZERO": -273.15
}


def rand_float(): return 2 ** randint(1, 12) - 1


def absolute_mirror(size=5, d=3):
    size = 4 if size < 4 else size
    h, r = int(size / 2), size % 2
    positives = sample(list(specials.values()), h - 1)
    positives.append(rand_float())
    positives = [abs(x) for x in positives]
    positives.sort()
    negatives = [-x for x in positives]
    negatives.reverse()
    if (r): negatives.append(0)
    return [round(x, d) for x in (negatives + positives)]



# copyright: hbr.org
# link: https://hbr.org/visual-library/2020/01/cultural-differences-in-assessing-trust-in-negotiating-partners

# x +: Interdependence
# x -: Independence
# y +: Flexibility
# y -: Stability
# head = ['interdependence', 'flexibility', 'culture']

IntegratedCultureFramework = [
    ['+1', '+2', 'Purpose'],
    ['+2', '+1', 'Caring'],
    ['+2', '-1', 'Order'],
    ['+1', '-2', 'Safety'],
    ['-1', '-2', 'Authority'],
    ['-2', '-1', 'Results'],
    ['-2', '+1', 'Enjoyment'],
    ['-1', '+2', 'Learning'],
]



from types import MethodType

from .arm_sales import arm_sales
from .brent_prices import brent_prices
from .mega_cities import mega_cities
from .mortality_rates import mortality_rates
from .power_cars import power_cars
from .recessions import recessions
from ...utils import FooDict
from ...utils.flops.dict_collection import flop_shuffle

dict_collection = FooDict({
    'ArmSales': arm_sales,
    'BrentPrices': brent_prices,
    'MegaCities': mega_cities,
    'MortalityRates': mortality_rates,
    'PowerCars': power_cars,
    'Recessions': recessions,
})

dict_collection.flop_shuffle = MethodType(flop_shuffle, dict_collection)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['foba',
 'foba.dicts',
 'foba.dicts.dict_numbers',
 'foba.dicts.dict_strings',
 'foba.matrices',
 'foba.matrices.matrix_numbers',
 'foba.matrices.matrix_numbers.functions',
 'foba.matrices.matrix_strings',
 'foba.objects',
 'foba.objects.point',
 'foba.quotes',
 'foba.tabulars',
 'foba.tabulars.crostabs',
 'foba.tabulars.tables',
 'foba.utils',
 'foba.utils.flops',
 'foba.utils.flops.dict_collection',
 'foba.utils.flops.vector_collection',
 'foba.utils.foo_dict',
 'foba.vectors',
 'foba.vectors.vector_numbers',
 'foba.vectors.vector_numbers.functions',
 'foba.vectors.vector_strings',
 'foba.vectors.vector_strings.utils']

package_data = \
{'': ['*']}

install_requires = \
['veho>=0.0.4', 'way>=0.0.1']

setup_kwargs = {
    'name': 'foba',
    'version': '0.0.7',
    'description': 'placeholder sample objects for test',
    'long_description': '# foba\n##### placeholder sample objects for test\n\n### Usage\n```python\nfrom foba.dicts import pastas\n\nprint(pastas)\n```',
    'author': 'Hoyeung Wong',
    'author_email': 'hoyeungw@outlook.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pydget/foba',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import random
from types import MethodType

from foba.utils import FooDict
from foba.vectors.vector_numbers.functions import \
    absolute_mirror, \
    compound_interest, \
    fibonacci, \
    hansen_samuelson, \
    leonardo, \
    primes

vector_collection = FooDict({
    'absolute_mirror': absolute_mirror,
    'compound_interest': compound_interest,
    'fibonacci': fibonacci,
    'hansen_samuelson': hansen_samuelson,
    'leonardo': leonardo,
    'primes': primes,
})


def flop_shuffle(self, length=3):
    k, fn = random.choice(list(self.items()))
    return k, fn(length)


vector_collection.flop_shuffle = MethodType(flop_shuffle, vector_collection)



from .dicts import number_dict_collection, string_dict_collection
from .matrices import number_matrix_collection, string_matrix_collection
from .objects import object_collection
from .tabulars import crostab_collection, table_collection
from .vectors import number_vector_collection, string_vector_collection



# https://www.sipri.org/databases/armsindustry
# in millions USD
arm_sales = {
    'LockheedMartin': 47260,
    'Boeing': 29150,
    'NorthropGrumman': 26190,
    'Raytheon': 23440,
    'GeneralDynamics': 22000,
    'BAESystemsUK': 21210,
    'AirbusGroup': 11650,
    'BAESystemsUS': 10800,
    'Leonardo': 9820,
    'AlmazAntey': 9640,
    'Thales': 9470,
    'UnitedTechnologies': 9310,
    'L3Technologies': 8250,
    'HuntingtonIngallsInd': 7200,
    'HoneywellIntr': 5430,
    'UnitedAircraft': 5420,
    'Leidos': 5000,
    'Harris': 4970,
    'UnitedShipbuilding': 4700,
    'BoozAllenHamilton': 4680,
    'RollsRoyce': 4680,
    'NavalGroup': 4220,
    'Rheinmetall': 3800,
    'MBDA': 3780,
    'GeneralElectric': 3650,
    'MitsubishiHeavyInd': 3620,
    'TacticalMissiles': 3600,
    'Textron': 3500,
    'ElbitSystems': 3500,
    'CACIIntr': 3490,
    'Saab': 3240,
    'Safran': 3240,
    'Sandia': 3200,
    'BabcockIntr': 3180,
    'UnitedEngine': 2950,
    'DassaultAviation': 2930,
    'ScienceApplicationsIntr': 2800,
    'AECOM': 2770,
    'GeneralAtomics': 2750,
    'HindustanAeronautics': 2740,
    'AirbusHelicoptersInc': 2730,
    'IsraelAerospaceInd': 2650,
    'HighPrecisionSystems': 2630,
    'RockwellCollins': 2630,
    'KBR': 2600,
    'Perspecta': 2590,
    'Rafael': 2540,
    'RussianElectronics': 2330,
    'HanwhaAerospace': 2320,
    'CEA': 2300,
    'KawasakiHeavyInd': 2260,
    'BellHelicopterTextron': 2030,
    'Bechtel': 2000,
    'Fincantieri': 1900,
    'Oshkosh': 1850,
    'RussianHelicopters': 1810,
    'KRET': 1770,
    'ASELSAN': 1740,
    'KraussMaffeiWegmann': 1680,
    'IndianOrdnanceFactories': 1650,
    'ThyssenKrupp': 1650,
    'Cobham': 1590,
    'DynCorpIntr': 1560,
    'KoreaAerospaceInd': 1550,
    'STEngineering': 1540,
    'BharatElectronics': 1460,
    'ManTechIntr': 1430,
    'UralVagonZavod': 1370,
    'JacobsEngineering': 1370,
    'Fluor': 1350,
    'LIGNex1': 1340,
    'TransDigm': 1330,
    'GKN': 1320,
    'MelroseInd': 1320,
    'UnitedLaunchAlliance': 1320,
    'UkrOboronProm': 1300,
    'Fujitsu': 1270,
    'SercoGroup': 1260,
    'PGZ': 1250,
    'TeledyneTechnologies': 1240,
    'Navantia': 1240,
    'Hensoldt': 1240,
    'Vectrus': 1230,
    'AerojetRocketdyne': 1220,
    'AustalUSA': 1160,
    'Austal': 1140,
    'SierraNevada': 1100,
    'IHI': 1090,
    'Nexter': 1080,
    'TurkishAerospaceInd': 1070,
    'BWXTechnologies': 1070,
    'Engility': 1070,
    'CAE': 1010,
    'MIT': 980,
    'Meggitt': 970,
    'CurtissWright': 970,
    'TheAerospace': 970,
    'DevonportRoyalDockyard': 940,
    'Ball': 930,
    'Moog': 920,
    'QinetiQ': 910,
    'RUAG': 900,
    'ViaSat': 860,
    'MitsubishiElectric': 860,
    'Arconic': 840,
    'NEC': 840,
    'Amphenol': 820,
}



def leonardo_generator():
    yield (prev := 1)
    yield (curr := 1)
    while True:
        prev, curr = curr, prev + curr + 1
        yield curr


def leonardo(n):
    gen = leonardo_generator()
    return [next(gen) for _ in range(n)]



pastas = {
  'conchiglie': 'seashell shaped, usually furrowed (rigate)',
  'chifferi': 'short and wide macaroni',
  'farfalle':'bow tie or butterfly shaped',
  'fusilli': 'long, thick, corkscrew-shaped pasta that may be solid or hollow',
  'gnocchi': 'lobed shells. Not to be confused with gnocchi dumplings',
  'lasagna': 'square or rectangle sheets of pasta',
  'linguine': 'flattened spaghetti',
  'pennine': 'medium length tubes with ridges, cut diagonally at both ends',
  'ravioli': 'wwo squares of pasta on top of another, stuffed with fill',
  'spaghetti': 'long, thin, cylindrical pasta of Italian origin'
}



# https://en.wikipedia.org/wiki/List_of_causes_of_death_by_rate
# data by deaths per 100,000

mortality_rates = {
  'Cardiovascular': 268.8,
  'InfectiousAndParasitic': 211.3,
  'CoronaryArtery': 115.8,
  'MalignantNeoplasms': 114.4,
  'Cerebrovascular': 88.5,
  'RespiratoryInfections': 63.7,
  'LowerRespiratoryTractInfections': 62.4,
  'Respiratory': 59.5,
  'UnintentionalInjuries': 57.0,
  'HIV': 44.6,
  'ChronicObstructivePulmonary': 44.1,
  'PerinatalConditions': 39.6,
  'Digestive': 31.6,
  'Diarrhea': 28.9,
  'IntentionalInjuries': 26.0,
  'Tuberculosis': 25.2,
  'Malaria': 20.4,
  'LungCancer': 20.0,
}
